#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import collections

import pygame
pygame.init()

from code.classes.Text import Text
from code.other.helpers import read_highscore


class Highscore_Table( pygame.sprite.Group ):
	"""
	A simple highscore table that can be blit onto a surface.  The 10 best scores a shown.
	"""

	def __init__( self, highscore_filename, center_xy_position, highscore_table_margin_bottom, font_size, font_color ):
		"""
		Parameters
			highscore_filename				string
				The name of the file that contains the data for the highscore.
			center_xy_position				( int, int ) or [ int, int ]
				Defines where to locate self.image on the surface where it is blitted onto.
			highscore_table_margin_bottom	int
				Defines the bottom margin between two highscore ranking sprites in px.
			font_size						int
				Defines the size of the font.
			font_color						int
				Defines the color of the font.
		"""
		pygame.sprite.Group.__init__( self )

		# I store some parameters that have been passed.
		self._center_xy_position = center_xy_position
		self._highscore_table_margin_bottom = highscore_table_margin_bottom
		self._font_size = font_size
		self._font_color = font_color

		# Load the highscore.
		self._highscore_dict = read_highscore( highscore_filename )

		# The highscore has been loaded.  It is now time to sort the entries in the dictionary.
		# The entry with the least time is the best player.
		self._ordered_highscores = self.sort_highscore_entries()

		ranking_counter = 1

		# Now create the single sprites that contain the information about a single highscore.
		for highscore_entry in self._ordered_highscores.itervalues():
			hs_text_to_blit = self.create_highscore_entry_text( ranking_counter, highscore_entry["name"], highscore_entry["score"] )

			hs_sprite = Text( hs_text_to_blit, self._center_xy_position, font_size=self._font_size, font_color=self._font_color )
			self.add( hs_sprite )

			# Some vertical space between two rankings.
			self._center_xy_position[1] += self._highscore_table_margin_bottom + hs_sprite.rect.height

			ranking_counter += 1


	def create_highscore_entry_text( self, place, player_name, time ):
		"""
		Creates the string that should be blitted to the surface later on.
		"""

		return "%d. %s %s" % ( place, player_name, time )


	def sort_highscore_entries( self ):
		"""
		This function sorts the entries of the highscores that has been loaded.
		It returns a collections.OrderedDict object.

		Sorting: The less time, the better the place / position.
		"""

		return collections.OrderedDict( sorted( self._highscore_dict.items(), key=self.test2 ) )


	def test2( self, val ):
		t = val[1]["score"].split(":")
		s = "".join(t)
		return s
