#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import math
import copy
import pygame
pygame.init()

from code.classes.Enemy import Enemy


class Kamikaze_Enemy( Enemy ):
	"""
	This enemy always moves towards the lunar rover, the player, and tries to damage / destroy it.
	By hitting the player, it destroys itself aswell.
	"""

	def __init__( self, image_file_names_enemy, animation_intervals_enemy, health_points, projectile_damage, name, left_top_position, speed, detection_distance ):
		"""
		Parameters
			image_file_names_enemy			( string, string, ... )
				These are the files names of the images, that should be used for the animation.
			animation_intervals_enemy		( int, int, ... )
				These are the intervals for the animation.  They define the update rate of the animation.
			health_points					int
				The health points of the enemy.
			projectile_damage				int
				How much damage does this enemy do, if it collides with the player ? ( Yes, the name is confusing.  This is due to bad class design. )
			name							string
				The name of the enemy.
			left_top_position				( int, int ) or [ int, int ]
				Where is the enemy located on the window / in the game ?
			speed							int
				Defines the speed of the enemy.
			detection_distance				int
				Defines the maximum distance towards the lunar rover, at which the kamikaze enemy should wake up and attack him.
		"""

		# Calling the base class with some parameters.
		super( Kamikaze_Enemy, self ).__init__( image_file_names_enemy, animation_intervals_enemy, health_points, projectile_damage, name, left_top_position, detection_distance )

		# Setting some variables.

		# The angle towards the lunar rover :-).
		self._direction_angle = 0 # Will be set throughout the game.
		self._speed = speed

	def update( self, lunar_rover_xy_velocity ):
		"""
		Parameters
			moon_rover_xy_velocity )
				Defines where the enemy should be moved if the player moves with his lunar rover.
		"""

		super( Kamikaze_Enemy, self ).update( lunar_rover_xy_velocity )		

		#self.rect.move_ip( moon_rover_xy_velocity[0], moon_rover_xy_velocity[1] )

		if not self._sleep:
			velocity_x, velocity_y = self.calculate_xy_velocity()
			self.rect.move_ip( velocity_x, velocity_y )


	def set_direction_angle( self, angle ):
		"""
		Sets the angle of where the enemy should go to.  Usually towards the player.

		Parameters
			angle			int
				The angle of where the kamikaze enemy should move to.
		"""

		self._direction_angle = angle


	def calculate_xy_velocity( self ):
		"""
		Calculates the velocity along the x and y axes.
		"""

		velocity_x = self._speed * math.sin( math.radians( self._direction_angle ) ) * -1
		velocity_y = self._speed * math.cos( math.radians( self._direction_angle ) ) * -1

		return velocity_x, velocity_y
