#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import pygame
pygame.init()

from code.classes.Text import Text


class Lunar_Rover_Sample_Meter( Text ):
	"""
	This class is the blueprint for the sample meter of the lunar rover.
	The sample meter shows how many samples have to be collected in the current level, and how many samples have already been collected in the current level.

	If all samples in a current level have been collected, then the level is completed and the user will move on to the next level.
	"""

	
	def __init__( self, samples_to_collect, samples_collected, samples_meter_text, center_xy_position ):
		"""
		Parameters
			samples_to_collect			int
				How many samples have to be collected in a level?
			samples_collected			int
				How many samples have already been collected in a level?
			samples_meter_text			string
				The unformatted text that should be displayed in the control panel of the Lunar Rover 47.
			center_xy_position			( int, int ) or [ int, int ]
				Defines where the sprite (the text) should be displayed on the window.
		"""

		
		# Store the parameters that have been passed.
		self._samples_to_collect = samples_to_collect
		self._samples_collected = samples_collected
		self._samples_meter_text = samples_meter_text
		self._center_xy_position = center_xy_position

		# Format the text so it can be displayed on the window.
		self._samples_meter_text_formatted = samples_meter_text.format( self._samples_collected, self._samples_to_collect )

		# Call the constructor of the base class.
		super( Lunar_Rover_Sample_Meter, self ).__init__( self._samples_meter_text_formatted, self._center_xy_position )


	def update_text( self ):
		"""
		Updates the status of the samples meter.
		"""

		# Format the text so it can be displayed on the window.
		self._samples_meter_text_formatted = self._samples_meter_text.format( self._samples_collected, self._samples_to_collect )

		# And update it of course, otherwise it won't be displayed.
		self.update( self._samples_meter_text_formatted )


	def increase_meter( self, n_samples ):
		"""
		Increases the amount of samples that have been collected by "n_samples".

		Parameters
			n_samples	int
				The amount of samples that have been collected.
		"""

		self._samples_collected += n_samples
		self.update_text()
