#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import pygame
pygame.init()

from code.other.helpers import load_image
from code.classes.Menu_Button import Menu_Button


class Menu( pygame.sprite.Sprite ):
	"""
	This class is the blue print for a game menu in this game.
	"""

	def __init__( self, background_image_filename, buttons ):
		"""
		Parameters
			background_image_filename	string
				The filename of the background image to use for this menu.
			buttons						dict
				A dictionary that contains information for the buttons that should be used in conjunction with this menu.
		"""
		
		pygame.sprite.Sprite.__init__( self )

		# I define the sprite.
		self.image = load_image( background_image_filename )
		self.rect = self.image.get_rect( topleft = ( 0, 0 ) ) # A background image always is located in the top-left corner of the window.

		# I create the buttons and store them in a sprite group.
		self._buttons = pygame.sprite.Group()
		for item in buttons.itervalues():
			self._buttons.add( Menu_Button( **item ) )

		# And I blit the buttons on self.image.  This will then be blitted onto "playground".
		self.blit_buttons()


	def blit_buttons( self ):
		"""
		Blits the buttons of the menu onto the background image of the menu (self.image, that is).
		"""

		self._buttons.draw( self.image )


	def update( self ):
		"""
		Updates the menu.

		This function updates and draws the buttons of the menu onto self.image, which will be blitted to the main window in the main-game loop.

		This function checks, if a button has been clicked by iterating through self._buttons.  If a button has been clicked, it's variable "b._clicked" is set to True.
		This means, that the value action value of the button ( b._action ) should be returned from this function to the main-game loop.
		"""

		self._buttons.update()
		self.blit_buttons()

		for b in self._buttons:
			if b._clicked:
				return b._action
