#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import pygame
pygame.init()

from code.other import constants, helpers

class Scrolling_Ground( pygame.sprite.Sprite ):
	"""
	A scrolling background that represents the different grounds of the moon.
	This is where the lunar rover moves on.
	"""


	def __init__( self, image_filename, left_top_start_pos, scrolling_speed, window_width, window_height ):
		"""
		Parameters
			image_filename		string
				The name of the image to load.
			left_top_start_pos	( int, int )
				The location of where to place the image.
			scrolling_speed		int
				How fast the background scrolls if the player is moving with his lunar rover.
			window_width
				The width of the window in pixels.
			window_height
				The height of the window in pixels.
		"""
		pygame.sprite.Sprite.__init__( self )

		# I save the parameters, as I might need them again later on.
		self._image_filename = image_filename
		self._left_top_start_pos = left_top_start_pos
		self._scrolling_speed = scrolling_speed
		self._window_width = window_width
		self._window_height = window_height

		# Setting up the sprite.
		self.image = helpers.load_image( image_filename )
		self.rect = self.image.get_rect( topleft = left_top_start_pos )


	def update( self, lunar_rover_velocity ):
		self.rect.move_ip( lunar_rover_velocity )
		self.relocate()


	def relocate( self ):
		"""
		If the ground has moved out of the screen, relocate it on the opposite side, just out of the screen.
		This ensures that the ground seems to be "endless".
		"""

		if self.rect.bottom <= 0:
			self.rect.top = self._window_height - self._scrolling_speed

		if self.rect.top > self._window_height:
			self.rect.bottom = self._scrolling_speed

		if self.rect.left > self._window_width:
			self.rect.right = self._scrolling_speed

		if self.rect.right < 0:
			self.rect.left = self._window_width - self._scrolling_speed
