#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import os.path

import pygame
pygame.init()

from code.other.constants import FONTS_DIR_PATH, FONT_FILENAME, DEFAULT_FONT_SIZE, DEFAULT_FONT_COLOR


class Text( pygame.sprite.Sprite ):
	"""
	This is the text class.  It is used to create information of text, which can then be displayed like a normal sprite, since the class inherits from 'pygame.sprite.Sprite'.
	The text of this class can be changed dynamically during the game, and thus, making it a bit more useful.
	"""

	def __init__( self, text, center_xy_position, font_filename=FONT_FILENAME, font_size=DEFAULT_FONT_SIZE, font_color=DEFAULT_FONT_COLOR, antialias=True ):
		"""
		Parameters
			text				string
				The text to be rendered onto self.image.
			center_xy_position	( int, int ) or [ int, int ]
				The position of where to display self.image on the window.
			font_filename		string
				The name of the font to be used for rendering the text onto self.image.
			font_size			int
				The size of the font to be used for rendering the text onto self.image.
			font_color			pygame.Color
				The color of the font that is used to render the text onto self.image.
			antialias			boolean
				If true, then the text, that is rendered onto self.image is antialiased.  Otherwise, the text won't be antialiased
		"""
		pygame.sprite.Sprite.__init__( self )

		# I store some parameters, as I need some later on again.
		self._center_xy_position = center_xy_position
		self._text = text
		self._text_old = self._text
		self._antialias = antialias
		self._font_color = font_color

		# I create the pygame.font.Font object.
		self._font = pygame.font.Font( os.path.join( FONTS_DIR_PATH, FONT_FILENAME ), font_size )
		
		# Draw the text onto the surface.
		self.image = self._font.render( self._text, self._antialias, self._font_color )
		self.rect = self.image.get_rect( center = self._center_xy_position )


	def update( self, text ):
		"""
		Updates the text if necessary - i.e. re-renders it if necesssary.
		"""
		
		if self.check_if_update_text( text ):
			self.render_new_text( text )


	def check_if_update_text( self, text ):
		"""
		Returns 'True' if a new text should be rendered onto self.image.
		"""

		if text != self._text_old:
			return True
		return False

		
	def render_new_text( self, text ):
		"""
		Renders a new text onto self.image.
		"""

		self._text = text
		self._text_old = text

		self.image = self._font.render( self._text, self._antialias, self._font_color )
		self.rect = self.image.get_rect( center = self._center_xy_position )

		
