#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules.
import os.path
import pygame
pygame.init()

# Display constants.
WINDOW_WIDTH = 640
WINDOW_HEIGHT = 480
WINDOW_DIMENSIONS = ( WINDOW_WIDTH, WINDOW_HEIGHT )

# Game clock constants.
CLOCK_FPS = 30

# Colors
RED = pygame.Color( 255, 0, 0, 0 )
BLUE = pygame.Color( 0, 0, 255, 0 )
YELLOW = pygame.Color( 255, 255, 0, 0 )
GREEN = pygame.Color( 0, 255, 0, 0 )
BLACK = pygame.Color( 0, 0, 0, 0 )
WHITE = pygame.Color( 255, 255, 255, 0 )
PURPLE = pygame.Color( 255, 0, 255, 0 )
DARK_BLUE = pygame.Color( 26, 54, 128, 255 )

# Data paths to images, sounds, ...
IMAGES_DIR_PATH = os.path.join( os.getcwd(), "data", "images" )
SOUNDS_DIR_PATH = os.path.join( os.getcwd(), "data", "sounds" )
MUSIC_DIR_PATH = os.path.join( os.getcwd(), "data", "music" )
LEVELS_DIR_PATH = os.path.join( os.getcwd(), "data", "levels" )
FONTS_DIR_PATH = os.path.join( os.getcwd(), "data", "fonts" )
HIGHSCORES_DIR_PATH = os.path.join( os.getcwd(), "data", "highscores" )

# Global constants, affecting almost all game objects.
SCROLLING_SPEED = 5 # Defines how fast the game-objects move.

# Class - Scrolling_Ground
# The scrolling ground consists of a matrix of 2x2 background images to ensure endless background-scrolling.
SCROLLING_GROUND_LEFT_TOP_START_POSITIONS = [
	( 0, 0 ), ( 0, 480 ), ( 640, 0 ), ( 640, 480 ) ]


# Class - Lunar_Rover_Shot
LUNAR_ROVER_SHOT = {
	"image_file_names": ( "lunar_rover_shot_1.png", "lunar_rover_shot_2.png",  "lunar_rover_shot_3.png","lunar_rover_shot_4.png", "lunar_rover_shot_5.png", "lunar_rover_shot_6.png", "lunar_rover_shot_7.png", "lunar_rover_shot_8.png", "lunar_rover_shot_9.png", "lunar_rover_shot_10.png", "lunar_rover_shot_11.png", "lunar_rover_shot_12.png", "lunar_rover_shot_13.png", "lunar_rover_shot_14.png", "lunar_rover_shot_15.png", "lunar_rover_shot_16.png", "lunar_rover_shot_17.png", "lunar_rover_shot_18.png", "lunar_rover_shot_19.png", "lunar_rover_shot_20.png", "lunar_rover_shot_21.png" ),
	"animation_intervals": ( 2, 2 ),
	"center_start_pos": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
	"speed": 9,
	"window_dimensions": ( 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT ),
	"angle": 0
}


# Class - Lunar_Rover
LUNAR_ROVER_DATA = {
	"animation_image_file_names": ( "lunar_rover_6.png", "lunar_rover_7.png", "lunar_rover_8.png", "lunar_rover_9.png", "lunar_rover_10.png" ),
	"center_xy_start_pos": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
	"animation_intervals": ( 5, 50, 5 ),
	"keys": {
		pygame.K_LEFT: "turn_left",
		pygame.K_RIGHT: "turn_right",
		pygame.K_UP:	"drive_forward",
		pygame.K_DOWN: "drive_backward",
		pygame.K_SPACE:"shoot",
	},
	"speed": SCROLLING_SPEED,
	"action_states": {
		"turn_left": False,
		"turn_right": False,
		"drive_forward": False,
		"drive_backward": False,
		"shoot": False,
	},
	"angle": 0,
	"degree_amount": 5,
	"shot": LUNAR_ROVER_SHOT,
	"collision_rect_data": ( 0, 0, 25, 25 ),
	"vehicle_health_data": {
		"vehicle_health": "100",
		"vehicle_health_text": "{0} %",
		"center_xy_position": ( 550, 57 ),
		"font_size": 32,
		"vehicle_max_health": 100,
		"vehicle_min_health": 0
	},
	"sample_meter_data": {
		"samples_to_collect": 0, # Will be defined dynamically in the game.
		"samples_collected": 0,
		"samples_meter_text": "{0} / {1}",
		"center_xy_position": ( 585, 185 )
	},
	"energy_meter_data": {
		"max_energy": 100.00,
		"min_energy": 0.00,
		"current_energy": 100.00,
		"center_xy_position": ( 585, 140 ),
		"energy_meter_text": "{0:.2f} %",
		"energy_production": 0, # Will be defined dynamically in the game by the current level.
		"energy_increase_interval": 1
	},
	"energy_consumption": {
		"drive": 0.2,
		"shoot": 1.0,
		"radar": 0.1,
		"turn": 0.1
	},
	"energy_production": 0, # WIll be defined dynamically through the level,
	"shot_damage": 1,
	"sound_shot_filename": "lunar_rover_shot.ogg",
	"sound_sample_collected_filename": "sample_collected.ogg"
}


# Class - Lunar_Rover_Control_Panel
LUNAR_ROVER_CONTROL_PANEL_DATA = {
	"image_file_names": ( "lunar_rover_47_control_panel_1.png", "lunar_rover_47_control_panel_2.png", "lunar_rover_47_control_panel_3.png", "lunar_rover_47_control_panel_4.png" ),
	"animation_intervals": ( 10, 10, 10, 40 ),
	"top_right_position": ( 640, 0 )
}


# Class - Radar
RADAR_DATA = {
	"dimensions": ( WINDOW_WIDTH / 4, WINDOW_HEIGHT / 4 ),
	"bottom_right_position": ( WINDOW_WIDTH - 7, WINDOW_HEIGHT - 10 ),
	"tracking_area": ( -WINDOW_WIDTH / 2, -WINDOW_HEIGHT / 2, WINDOW_WIDTH * 2, WINDOW_HEIGHT * 2 ),
	"color": RED,
	"transparency_color": BLACK,
	"keys": {
		pygame.K_r: "turn_radar_off_on"
	},
	"action_states": {
		"turn_radar_off_on": False
	},
	"radar_on_text": "on",
	"radar_on_text_center_xy": ( 584, 361 ),
	"radar_off_text": "off",
	"radar_off_text_center_xy": ( 584, 361 ),
	"radar_info_text_font_size": 11
}


# Class - Level_Clock
CLOCK_DATA = {
	"mins": 0,
	"secs": 0,
	"second_in_frames": CLOCK_FPS,
	"clock_text": "{0:0=2}:{1:0=2}",
	"center_xy_position": ( 585, 233 )
}


# Enemy classes
ENEMY_TOWER_1 = {
	"image_file_names_enemy": ( "enemy_tower_1_1.png", "enemy_tower_1_2.png", "enemy_tower_1_3.png", "enemy_tower_1_4.png" ),
	"animation_intervals_enemy": ( 5, 7, 13 ),
	"health_points": 3,
	"projectile_damage": 1,
	"name": "TB 0129 -2",
	"left_top_position": (  ), # Will be defined through the map of a level.  It depends where it is placed in there.
	"shooting_interval": 60, # Frames
	"shot_data": {
		"image_file_names": ("tower_shot_1.png", "tower_shot_2.png", "tower_shot_3.png", "tower_shot_4.png", "tower_shot_5.png", "tower_shot_6.png", "tower_shot_7.png", "tower_shot_8.png", "tower_shot_9.png", "tower_shot_10.png"),
		"animation_intervals": ( 3, 3 ),
		"center_start_pos": ( 0, 0 ),
		"speed": 8,
		"window_dimensions": ( 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT ),
		"angle": 0 # Will be set dynamically during the game.
	},
	"detection_distance": 250
}


ENEMY_TOWER_2 = {
	"image_file_names_enemy": ( "enemy_tower_1_1.png", "enemy_tower_1_2.png", "enemy_tower_1_3.png", "enemy_tower_1_4.png" ),
	"animation_intervals_enemy": ( 2, 2 ),
	"health_points": 4,
	"projectile_damage": 2,
	"name": "TB 0129",
	"left_top_position": (  ), # Will be defined through the map of a level.  It depends where it is placed in there.
	"shooting_interval": 60, # Frames
	"shot_data": {
		"image_file_names": ("tower_shot_1.png", "tower_shot_2.png", "tower_shot_3.png", "tower_shot_4.png", "tower_shot_5.png", "tower_shot_6.png", "tower_shot_7.png", "tower_shot_8.png", "tower_shot_9.png", "tower_shot_10.png"),
		"animation_intervals": ( 3, 3 ),
		"center_start_pos": ( 0, 0 ),
		"speed": 10,
		"window_dimensions": ( 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT ),
		"angle": 0 # Will be set dynamically during the game.
	},
	"detection_distance": 250
}


ENEMY_TOWER_3 = {
	"image_file_names_enemy": ( "enemy_tower_1_1.png", "enemy_tower_1_2.png", "enemy_tower_1_3.png", "enemy_tower_1_4.png" ),
	"animation_intervals_enemy": ( 6, 6 ),
	"health_points": 6,
	"projectile_damage": 4,
	"name": "TB 0129 -3",
	"left_top_position": (  ), # Will be defined through the map of a level.  It depends where it is placed in there.
	"shooting_interval": 40, # Frames
	"shot_data": {
		"image_file_names": ("tower_shot_1.png", "tower_shot_2.png", "tower_shot_3.png", "tower_shot_4.png", "tower_shot_5.png", "tower_shot_6.png", "tower_shot_7.png", "tower_shot_8.png", "tower_shot_9.png", "tower_shot_10.png"),
		"animation_intervals": ( 3, 3 ),
		"center_start_pos": ( 0, 0 ),
		"speed": 12,
		"window_dimensions": ( 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT ),
		"angle": 0 # Will be set dynamically during the game.
	},
	"detection_distance": 250
}


ENEMY_KAMIKAZE_1 = {
	"image_file_names_enemy": ( "kamikaze_enemy_1_1.png", "kamikaze_enemy_1_2.png", "kamikaze_enemy_1_3.png", "kamikaze_enemy_1_4.png", "kamikaze_enemy_1_5.png", "kamikaze_enemy_1_6.png", "kamikaze_enemy_1_7.png", "kamikaze_enemy_1_8.png", "kamikaze_enemy_1_9.png", "kamikaze_enemy_1_10.png", "kamikaze_enemy_1_11.png", "kamikaze_enemy_1_12.png", "kamikaze_enemy_1_13.png", "kamikaze_enemy_1_14.png","kamikaze_enemy_1_15.png", "kamikaze_enemy_1_16.png", "kamikaze_enemy_1_17.png", "kamikaze_enemy_1_18.png", "kamikaze_enemy_1_19.png", "kamikaze_enemy_1_20.png", "kamikaze_enemy_1_21.png", "kamikaze_enemy_1_22.png", "kamikaze_enemy_1_23.png", "kamikaze_enemy_1_24.png", "kamikaze_enemy_1_25.png", "kamikaze_enemy_1_26.png", "kamikaze_enemy_1_27.png", "kamikaze_enemy_1_28.png", "kamikaze_enemy_1_29.png", "kamikaze_enemy_1_30.png", "kamikaze_enemy_1_31.png", "kamikaze_enemy_1_32.png" ),
	"animation_intervals_enemy": ( 2, 2 ),
	"health_points": 2,
	"projectile_damage": 5,
	"name": "K 557 efy",
	"left_top_position": ( 0, 0 ), # Will be defined through the map of a level.
	"speed": 3,
	"detection_distance": 150
}


ENEMY_KAMIKAZE_2 = {
	"image_file_names_enemy": ( "kamikaze_enemy_1_1.png", "kamikaze_enemy_1_2.png", "kamikaze_enemy_1_3.png", "kamikaze_enemy_1_4.png", "kamikaze_enemy_1_5.png", "kamikaze_enemy_1_6.png", "kamikaze_enemy_1_7.png", "kamikaze_enemy_1_8.png", "kamikaze_enemy_1_9.png", "kamikaze_enemy_1_10.png", "kamikaze_enemy_1_11.png", "kamikaze_enemy_1_12.png", "kamikaze_enemy_1_13.png", "kamikaze_enemy_1_14.png","kamikaze_enemy_1_15.png", "kamikaze_enemy_1_16.png", "kamikaze_enemy_1_17.png", "kamikaze_enemy_1_18.png", "kamikaze_enemy_1_19.png", "kamikaze_enemy_1_20.png", "kamikaze_enemy_1_21.png", "kamikaze_enemy_1_22.png", "kamikaze_enemy_1_23.png", "kamikaze_enemy_1_24.png", "kamikaze_enemy_1_25.png", "kamikaze_enemy_1_26.png", "kamikaze_enemy_1_27.png", "kamikaze_enemy_1_28.png", "kamikaze_enemy_1_29.png", "kamikaze_enemy_1_30.png", "kamikaze_enemy_1_31.png", "kamikaze_enemy_1_32.png" ),
	"animation_intervals_enemy": ( 5, 5 ),
	"health_points": 3,
	"projectile_damage": 7,
	"name": "K 557 efy-2",
	"left_top_position": ( 0, 0 ), # Will be defined through the map of a level.
	"speed": 4,
	"detection_distance": 170
}


ENEMY_KAMIKAZE_3 = {
	"image_file_names_enemy": ( "kamikaze_enemy_1_1.png", "kamikaze_enemy_1_2.png", "kamikaze_enemy_1_3.png", "kamikaze_enemy_1_4.png", "kamikaze_enemy_1_5.png", "kamikaze_enemy_1_6.png", "kamikaze_enemy_1_7.png", "kamikaze_enemy_1_8.png", "kamikaze_enemy_1_9.png", "kamikaze_enemy_1_10.png", "kamikaze_enemy_1_11.png", "kamikaze_enemy_1_12.png", "kamikaze_enemy_1_13.png", "kamikaze_enemy_1_14.png","kamikaze_enemy_1_15.png", "kamikaze_enemy_1_16.png", "kamikaze_enemy_1_17.png", "kamikaze_enemy_1_18.png", "kamikaze_enemy_1_19.png", "kamikaze_enemy_1_20.png", "kamikaze_enemy_1_21.png", "kamikaze_enemy_1_22.png", "kamikaze_enemy_1_23.png", "kamikaze_enemy_1_24.png", "kamikaze_enemy_1_25.png", "kamikaze_enemy_1_26.png", "kamikaze_enemy_1_27.png", "kamikaze_enemy_1_28.png", "kamikaze_enemy_1_29.png", "kamikaze_enemy_1_30.png", "kamikaze_enemy_1_31.png", "kamikaze_enemy_1_32.png" ),
	"animation_intervals_enemy": ( 5, 5 ),
	"health_points": 5,
	"projectile_damage": 8,
	"name": "K 557 efy-3",
	"left_top_position": ( 0, 0 ), # Will be defined through the map of a level.
	"speed": 6,
	"detection_distance": 190
}


ENEMY_ROVER_1 = {
	"image_file_names_enemy": ( "moving_enemy_1_1.png", "moving_enemy_1_2.png" ),
	"animation_intervals_enemy": ( 2, 2 ),
	"health_points": 4,
	"projectile_damage": 1,
	"name": "R 74 Tea-F",
	"left_top_position": ( 0, 0 ), # Just a placeholder.  This will be set by the map of a level.
	"shooting_interval": 70, # Frames
	"moving_speed": 3, # The speed of the moving enemy.
	"shot_data": {
		"image_file_names": ( "moving_enemy_shot_1.png", "moving_enemy_shot_2.png", "moving_enemy_shot_3.png", "moving_enemy_shot_4.png", "moving_enemy_shot_5.png", "moving_enemy_shot_6.png", "moving_enemy_shot_7.png", "moving_enemy_shot_8.png", "moving_enemy_shot_9.png", "moving_enemy_shot_10.png", ),
		"animation_intervals": ( 5, 5 ),
		"center_start_pos": ( 0, 0 ),
		"speed": 6,
		"window_dimensions": ( 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT ),
		"angle": 0 # Will be set dynamically during the game.
	},
	"min_distance": 190,
	"detection_distance": 300 # Defines the distance at which the enemy should begin to shoot and move.	
}


ENEMY_ROVER_2 = {
	"image_file_names_enemy": ( "moving_enemy_1_1.png", "moving_enemy_1_2.png" ),
	"animation_intervals_enemy": ( 7, 7 ),
	"health_points": 6,
	"projectile_damage": 2,
	"name": "R 74 Tea-F-2",
	"left_top_position": ( 0, 0 ), # Just a placeholder.  This will be set by the map of a level.
	"shooting_interval": 50, # Frames
	"moving_speed": 5, # The speed of the moving enemy.
	"shot_data": {
		"image_file_names": ( "moving_enemy_shot_1.png", "moving_enemy_shot_2.png", "moving_enemy_shot_3.png", "moving_enemy_shot_4.png", "moving_enemy_shot_5.png", "moving_enemy_shot_6.png", "moving_enemy_shot_7.png", "moving_enemy_shot_8.png", "moving_enemy_shot_9.png", "moving_enemy_shot_10.png", ),
		"animation_intervals": ( 3, 3 ),
		"center_start_pos": ( 0, 0 ),
		"speed": 6,
		"window_dimensions": ( 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT ),
		"angle": 0 # Will be set dynamically during the game.
	},
	"min_distance": 170,
	"detection_distance": 350 # Defines the distance at which the enemy should begin to shoot and move.	
}


ENEMY_ROVER_3 = {
	"image_file_names_enemy": ( "moving_enemy_1_1.png", "moving_enemy_1_2.png" ),
	"animation_intervals_enemy": ( 4, 4 ),
	"health_points": 8,
	"projectile_damage": 3,
	"name": "R 74 Tea-F-3",
	"left_top_position": ( 0, 0 ), # Just a placeholder.  This will be set by the map of a level.
	"shooting_interval": 40, # Frames
	"moving_speed": 5, # The speed of the moving enemy.
	"shot_data": {
		"image_file_names": ( "moving_enemy_shot_1.png", "moving_enemy_shot_2.png", "moving_enemy_shot_3.png", "moving_enemy_shot_4.png", "moving_enemy_shot_5.png", "moving_enemy_shot_6.png", "moving_enemy_shot_7.png", "moving_enemy_shot_8.png", "moving_enemy_shot_9.png", "moving_enemy_shot_10.png", ),
		"animation_intervals": ( 4, 4 ),
		"center_start_pos": ( 0, 0 ),
		"speed": 7,
		"window_dimensions": ( 0, 0, WINDOW_WIDTH, WINDOW_HEIGHT ),
		"angle": 0 # Will be set dynamically during the game.
	},
	"min_distance": 170,
	"detection_distance": 370 # Defines the distance at which the enemy should begin to shoot and move.	
}


# Data for all levels of the game.
LEVEL_DATA = {
	1: {
		"map_data": {
			"level_map_file": "level_1.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 1,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_1,
				"x": ENEMY_KAMIKAZE_1,
				"y": ENEMY_ROVER_1
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.20,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 1,
				"level_name": "Arrival",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "scrolling_ground_2.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 1 - Arrival",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 1 - Arrival",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	},
	2: {
		"map_data": {
			"level_map_file": "level_2.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 2,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_1,
				"x": ENEMY_KAMIKAZE_1,
				"y": ENEMY_ROVER_1
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.19,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 2,
				"level_name": "Samples",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "scrolling_ground_2.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 2 - Samples",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30			
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 2 - Samples",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	},
	3: {
		"map_data": {
			"level_map_file": "level_3.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 3,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_2,
				"x": ENEMY_KAMIKAZE_1,
				"y": ENEMY_ROVER_1
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.18,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 3,
				"level_name": "Danger!",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "snow_ground.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 3 - Danger!",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30			
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 3 - Danger!",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	},
	4: {
		"map_data": {
			"level_map_file": "level_4.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 4,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_2,
				"x": ENEMY_KAMIKAZE_1,
				"y": ENEMY_ROVER_1
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.17,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 4,
				"level_name": "Pssht!",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "snow_ground.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 4 - Pssht!",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30			
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 4 - Pssht!",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	},
	5: {
		"map_data": {
			"level_map_file": "level_5.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 5,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_2,
				"x": ENEMY_KAMIKAZE_1,
				"y": ENEMY_ROVER_1
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.16,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 5,
				"level_name": "Alone?",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "purple_ground.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 5 - Alone?",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30			
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 5 - Alone?",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	},
	6: {
		"map_data": {
			"level_map_file": "level_6.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 6,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_2,
				"x": ENEMY_KAMIKAZE_2,
				"y": ENEMY_ROVER_1
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.14,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 6,
				"level_name": "Singing",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "purple_ground.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 6 - Singing",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30			
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 6 - Singing",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	},
	7: {
		"map_data": {
			"level_map_file": "level_7.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 7,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_2,
				"x": ENEMY_KAMIKAZE_2,
				"y": ENEMY_ROVER_2
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.13,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 7,
				"level_name": "Energy",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "brown_ground.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 7 - Energy",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30			
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 7 - Energy",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	},
	8: {
		"map_data": {
			"level_map_file": "level_8.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 8,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_2,
				"x": ENEMY_KAMIKAZE_2,
				"y": ENEMY_ROVER_2
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.12,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 8,
				"level_name": "Nostalgia",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "brown_ground.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 8 - Nostalgia",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30			
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 8 - Nostalgia",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	},
	9: {
		"map_data": {
			"level_map_file": "level_9.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 9,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_2,
				"x": ENEMY_KAMIKAZE_3,
				"y": ENEMY_ROVER_2
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.11,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 9,
				"level_name": "Chilly",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "ice_ground.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 9 - Chilly",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30			
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 9 - Chilly",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	},
	10: {
		"map_data": {
			"level_map_file": "level_10.txt",
			"map_center_position": ( WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2 ),
			"lvl_nr": 10,
			"hash_table": {
				"*": {
					"image_filename": "lvl_1_sample_1.png",
					"left_top_position": ( 0, 0 )
				},
				"#": {
					"class": "None",
				},
				"+": ENEMY_TOWER_3,
				"x": ENEMY_KAMIKAZE_3,
				"y": ENEMY_ROVER_3
			},
			"tile_size": 40,
			"window_dimensions": ( WINDOW_WIDTH, WINDOW_HEIGHT ),
			"energy_production": 0.10,
			"level_clock_data": CLOCK_DATA,
			"level_meter_data": {
				"level": 10,
				"level_name": "Almost",
				"level_unformatted_text": "{0} - {1}",
				"center_xy_position": ( 585, 280 )
			}
		},
		"background_data": {
			"image_filename": "ice_ground.jpg",
			"scrolling_speed": SCROLLING_SPEED,
			"left_top_start_pos": ( 0, 0 ), # Just a placeholder.
			"window_width": WINDOW_WIDTH,
			"window_height": WINDOW_HEIGHT		
		},
		"transition_fade_in": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 10 - Almost",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"waiting_time": 30			
		},
		"transition_fade_out": {
			"window_dimensions": WINDOW_DIMENSIONS,
			"text": "Level 10 - Almost",
			"font_size": 40,
			"font_color": WHITE,
			"bg_color": BLACK,
			"transition_speed": 11,
			"fade_in":False
		}		
	}	
}


# class - Text
FONT_FILENAME = "Ubuntu-L.ttf"
DEFAULT_FONT_SIZE = 14
DEFAULT_FONT_COLOR = WHITE


# Constants / data for the menues.

# Main menu

MAIN_MENU_DATA = {
	"background_image_filename": ( "title_screen.png" ),
	"buttons": {
		"btn_start": {
			"btn_image_off_filename": "title_screen_start_btn_off.png",
			"btn_image_on_filename": "title_screen_start_btn_on.png",
			"action": "start_game",
			"left_top_position": ( 130, 185 )
		},
		"btn_help": {
			"btn_image_off_filename": "title_screen_help_btn_off.png",
			"btn_image_on_filename": "title_screen_help_btn_on.png",
			"action": "show_help",
			"left_top_position": ( 140, 205 )
		},
		"btn_prologue": {
			"btn_image_off_filename": "title_screen_prologue_btn_off.png",
			"btn_image_on_filename": "title_screen_prologue_btn_on.png",
			"action": "show_prologue",
			"left_top_position": ( 145, 225 )
		},
		"btn_highscore": {
			"btn_image_off_filename": "title_screen_highscores_btn_off.png",
			"btn_image_on_filename": "title_screen_highscores_btn_on.png",
			"action": "show_highscores",
			"left_top_position": ( 140, 245 )
		},
		"btn_exit": {
			"btn_image_off_filename": "title_screen_exit_btn_off.png",
			"btn_image_on_filename": "title_screen_exit_btn_on.png",
			"action": "exit_game",
			"left_top_position": ( 130, 265 )
		}
	}
}

# Help menu
HELP_MENU_DATA = {
	"background_image_filename": "help_screen.png",
	"buttons": {
		"btn_back": {
			"btn_image_off_filename": "back_btn_off.png",
			"btn_image_on_filename": "back_btn_on.png",
			"action": "back",
			"left_top_position": ( 130, 185 )
		}
	}
}


# Prologue menu
PROLOGUE_MENU_1_DATA = {
	"background_image_filename": "prologue_screen_1.png",
	"buttons": {
		"btn_back": {
			"btn_image_off_filename": "back_btn_off.png",
			"btn_image_on_filename": "back_btn_on.png",
			"action": "back",
			"left_top_position": ( 130, 185 )
		},
		"btn_next": {
			"btn_image_off_filename": "next_btn_off.png",
			"btn_image_on_filename": "next_btn_on.png",
			"action": "next",
			"left_top_position": ( 400, 460 )
		}	
	}
}


# Prologue menu
PROLOGUE_MENU_2_DATA = {
	"background_image_filename": "prologue_screen_2.png",
	"buttons": {
		"btn_back": {
			"btn_image_off_filename": "back_btn_off.png",
			"btn_image_on_filename": "back_btn_on.png",
			"action": "back",
			"left_top_position": ( 130, 185 )
		},
		"btn_next": {
			"btn_image_off_filename": "previous_btn_off.png",
			"btn_image_on_filename": "previous_btn_on.png",
			"action": "previous",
			"left_top_position": ( 400, 460 )
		}	
	}
}


# Highscore menu
HIGHSCORE_MENU_DATA = {
	"background_image_filename": "highscore_screen.png",
	"buttons": {
		"btn_back": {
			"btn_image_off_filename": "back_btn_off.png",
			"btn_image_on_filename": "back_btn_on.png",
			"action": "back",
			"left_top_position": ( 130, 185 )
		}
	}
}


MAIN_MENU_TRANSITION = {
	"transition_fade_in": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 30,
	},
	"transition_fade_out": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 30,
		"fade_in":False
	}
}

HELP_MENU_TRANSITION = {
	"transition_fade_in": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 30
	},
	"transition_fade_out": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 30,
		"fade_in":False
	}
}

GAME_OVER_TRANSITION = {
	"transition_fade_in": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "Game over!",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 11,
		"waiting_time": 30
	},
	"transition_fade_out": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "Game over!",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 11,
		"fade_in":False
	}
}


GAME_COMPLETED_TRANSITION = {
	"transition_fade_in": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "Congratulations!  Return to Earth.",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 11,
		"waiting_time": 30
	},
	"transition_fade_out": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "Congratulations!  Return to Earth.",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 11,
		"fade_in":False
	}
}


ENTER_NAME_TRANSITION = {
	"transition_fade_in": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "Please enter your name:",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 16,
	},
	"transition_fade_out": {
		"window_dimensions": WINDOW_DIMENSIONS,
		"text": "Please enter your name:",
		"font_size": 40,
		"font_color": WHITE,
		"bg_color": BLACK,
		"transition_speed": 16,
		"fade_in":False
	}
}


# Data for the highscore class
HIGHSCORE_DATA = {
	"highscore_filename": "highscore.txt",
	"center_xy_position": [ 367, 195 ],
	"highscore_table_margin_bottom": 10,
	"font_size": 14,
	"font_color": DARK_BLUE
}
# For testing purposes
TEST_OBJECTS_DATA = ( ( 100, 100 ), ( 450, 230 ), ( 630, 470 ), ( 603, 388 ) )
