#    Lunar Rover Expedition - A fun adventure game.
#    Copyright (C) 2013 Jannik Haberbosch
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Importion of modules
import datetime
import copy
import pygame
pygame.init()

from code.other import constants, helpers
from code.classes.Scrolling_Ground import Scrolling_Ground
from code.classes.Lunar_Rover import Lunar_Rover
from code.classes.Lunar_Rover_Control_Panel import Lunar_Rover_Control_Panel
from code.classes.Radar import Radar
from code.classes.Test_Object import Test_Object
from code.classes.Level import Level
from code.classes.Transition import Transition
from code.classes.Menu import Menu
from code.classes.Highscore_Table import Highscore_Table
from code.classes.Text import Text


# The main function.  The game starts here.
def main():

	# Set up the window
	playground = pygame.display.set_mode( constants.WINDOW_DIMENSIONS )

	# Set up the game-clock.
	game_clock = pygame.time.Clock()

	# Display the main menu
	show_main_menu( playground, game_clock )
	

def next_level( playground, game_clock, lvl_nr, player_time, vehicle_health ):
	"""
	Setup and run the next level.

	Parameters
		lvl_nr			int
			The number of the next level.
	"""

	# A variable that is set to True if the lunar rover has been destroyed.
	lunar_rover_destroyed = False
	
	# Transition fade-in.
	helpers.run_transition( playground, game_clock, constants.CLOCK_FPS, constants.LEVEL_DATA[lvl_nr]["transition_fade_in"] )

	# Load the scrolling ground.
	scrolling_ground = pygame.sprite.Group()
	for x in constants.SCROLLING_GROUND_LEFT_TOP_START_POSITIONS:
		constants.LEVEL_DATA[lvl_nr]["background_data"]["left_top_start_pos"] = x
		scrolling_ground.add( Scrolling_Ground( **constants.LEVEL_DATA[lvl_nr]["background_data"] ) )

	# Load the first level
	level = Level( **constants.LEVEL_DATA[lvl_nr]["map_data"] )

	# Load the lunar rover.
	# Before I create the object, I have to set the amount of samples that have to be collected in the current level.
	constants.LUNAR_ROVER_DATA["sample_meter_data"]["samples_to_collect"] = level._n_samples_to_collect
	# I have to set the solar panel energy interval rate aswell.
	constants.LUNAR_ROVER_DATA["energy_meter_data"]["energy_production"] = level._energy_production

	# I update the health of the vehicle.  This means, that a previously damaged lunar rover will stil be damaged in the next level, and not be repaired.
	constants.LUNAR_ROVER_DATA["vehicle_health_data"]["vehicle_health"] = int( vehicle_health )
	lunar_rover = Lunar_Rover( **constants.LUNAR_ROVER_DATA )

	# Load the shadow.
	shadow_image = helpers.load_image( "shadow.png" )
	shadow_rect = shadow_image.get_rect( topleft = ( 0, 0 ) )

	# Load the lunar rover control panel
	lunar_rover_control_panel = Lunar_Rover_Control_Panel( **constants.LUNAR_ROVER_CONTROL_PANEL_DATA )

	# Set up the terrain radar
	terrain_radar = Radar( **constants.RADAR_DATA )
	# And add a reference of the terrain radar to the lunar rover.  NOTE TO SELF:  No good solution!
	lunar_rover._terrain_radar_ref = terrain_radar


	# I blit all game-objects (sprites) onto a temporary sprite of the size of the window.  This will be needed for the fade-out transition.
	temp_sf = pygame.sprite.Sprite()
	temp_sf.image = pygame.Surface( constants.WINDOW_DIMENSIONS )
	temp_sf.rect = temp_sf.image.get_rect( topleft = ( 0, 0 ) )

	# Draw everything onto this temporary surface of the sprite.
	scrolling_ground.draw( temp_sf.image )
	level._all_level_objects.draw( temp_sf.image )
	temp_sf.image.blit( lunar_rover.image, lunar_rover.rect )
	for shooting_enemy in level._shooting_enemies:
		shooting_enemy._fired_shots.draw( temp_sf.image )
	lunar_rover._shots.draw( temp_sf.image )
	temp_sf.image.blit( shadow_image, shadow_rect )
	temp_sf.image.blit( lunar_rover_control_panel.image, lunar_rover_control_panel.rect )
	temp_sf.image.blit( terrain_radar.image, terrain_radar.rect )
	temp_sf.image.blit( lunar_rover._vehicle_health.image, lunar_rover._vehicle_health.rect )
	temp_sf.image.blit( lunar_rover._sample_meter.image, lunar_rover._sample_meter.rect )
	temp_sf.image.blit( lunar_rover._energy_meter.image, lunar_rover._energy_meter.rect )
	temp_sf.image.blit( terrain_radar._radar_info_text.image, terrain_radar._radar_info_text.rect )
	temp_sf.image.blit( level._level_clock.image, level._level_clock.rect )
	temp_sf.image.blit( level._level_meter.image, level._level_meter.rect )	

	# Now create and run the fade-out transition.
	helpers.run_transition( playground, game_clock, constants.CLOCK_FPS, constants.LEVEL_DATA[lvl_nr]["transition_fade_out"], temp_sf )
		
	
	# Run the game.
	running = True
	while running:
		# Fill the background with a standard black color.
		playground.fill( constants.BLACK )

		# User-quit check.
		events = pygame.event.get()
		pressed_keys = pygame.key.get_pressed()
		if pressed_keys[pygame.K_ESCAPE]:
			running = False
			# The player has aborted the game.
			lunar_rover_destroyed = True

		# Update everyting.
		scrolling_ground.update( lunar_rover._velocity )
		level._all_level_objects.update( lunar_rover._velocity )
		level.update_level_clock()
		if lunar_rover.update( pressed_keys ):
			# The lunar rover has been destroyed.  End this game.
			running = False
			lunar_rover_destroyed = True
		lunar_rover_control_panel.update()
		terrain_radar.update( level._all_level_objects, pressed_keys )

		helpers.calc_angle_between_obstacles_and_player_lunar_rover( lunar_rover, level._obstacles )
		helpers.collision_lunar_rover_shots_and_enemies_shots( lunar_rover, level._obstacles, level._shooting_enemies )

		helpers.calc_distance_moving_enemy_lunar_rover( level._obstacles, lunar_rover )

		# Check for collisions.
		helpers.collision_lunar_rover_level_objects( lunar_rover, level._all_level_objects )

		# Check if all samples in the current level have been collected.
		if level.all_samples_collected():
			running = False

		# Draw everything.
		scrolling_ground.draw( playground )
		level._all_level_objects.draw( playground )
		playground.blit( lunar_rover.image, lunar_rover.rect )
		for shooting_enemy in level._shooting_enemies:
			shooting_enemy._fired_shots.draw( playground )
		lunar_rover._shots.draw( playground )
		playground.blit( shadow_image, shadow_rect )
		playground.blit( lunar_rover_control_panel.image, lunar_rover_control_panel.rect )
		playground.blit( terrain_radar.image, terrain_radar.rect )
		playground.blit( lunar_rover._vehicle_health.image, lunar_rover._vehicle_health.rect )
		playground.blit( lunar_rover._sample_meter.image, lunar_rover._sample_meter.rect )
		playground.blit( lunar_rover._energy_meter.image, lunar_rover._energy_meter.rect )
		playground.blit( terrain_radar._radar_info_text.image, terrain_radar._radar_info_text.rect )
		playground.blit( level._level_clock.image, level._level_clock.rect )
		playground.blit( level._level_meter.image, level._level_meter.rect )


		# Update everything on the display.
		pygame.display.flip()

		# Run the game at a specified frame-rate.
		game_clock.tick( constants.CLOCK_FPS )


	# This level is over.  Now I add the time, which the user required for this level, to the overall time of the user.
	mins = level._level_clock._mins
	secs = level._level_clock._secs
	
	player_time = player_time + datetime.timedelta( 0, secs, 0, 0, mins )

	if lunar_rover_destroyed:
		return "lunar_rover_destroyed", player_time, lunar_rover._vehicle_health._vehicle_health
	else:
		return False, player_time, lunar_rover._vehicle_health._vehicle_health

def run_game( playground, clock ):
	"""
	Runs the game.
	"""

	# How many levels are there?
	n_levels = len( constants.LEVEL_DATA )

	# The current level.
	current_level = 1

	# This function can return a value that tells the main menu what to do if the player returns from the game.
	game_return_value = ""

	# I am going to track the time of the player.  If the player is good, he might make it into the highscore / ranking table.
	player_time = datetime.datetime( 1901, 1, 1, 0, 0, 0 )

	# A damaged car won't heal between levels.  Otherwise, the game would not be so challenging.
	vehicle_health = "100"

	running = True
	while running:
		
		# status contains a value that tells, whether the lunar rover has been destroyed or not, or whether the user wanted to just quit the game.
		status, player_time, vehicle_health = next_level( playground, clock, current_level, player_time, vehicle_health )

		# Has the lunar rover been destroyed?
		if status == "lunar_rover_destroyed":
			# It has.  So the game is over.
			# Run a fade-in transition with the message "Game Over".
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.GAME_OVER_TRANSITION["transition_fade_in"] )

			# And go back to the main menu with a string value.
			running = False
			game_return_value = "return_from_game_over"
			return game_return_value, player_time
			

		# Check if there is another level.
		if current_level + 1 <= n_levels:
			current_level += 1
		else:
			running = False

			# The player has won the game!  Write a message to the screen which goes like "Completed".
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.GAME_COMPLETED_TRANSITION["transition_fade_in"] )
						
			return "return_game_won", player_time

def show_main_menu( playground, clock ):
	"""
	Displays the main menu of the game.
	"""

	# Create the main menu.
	main_menu = Menu( **constants.MAIN_MENU_DATA )

	# Run the transition.
	helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.MAIN_MENU_TRANSITION["transition_fade_in"] )
	helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.MAIN_MENU_TRANSITION["transition_fade_out"], main_menu )	

	while True:
		events = pygame.event.get()

		# Update the main menu.  'action' will contain a string value, if a button of the menu has been pressed.
		action = main_menu.update()

		# And draw the menu.
		playground.blit( main_menu.image, main_menu.rect )

		# Flip the screen, so it is visible.
		pygame.display.flip()

		# Run everything at a specified frame-rate.
		clock.tick( constants.CLOCK_FPS )

		# Now check which button the user has clicked.  If the user didn't click any button, then the entire block will simply be ignored.
		if action == "start_game":
			# Start a new game.

			# First, ask the user for his / her name.
			player_name, player_name_sprite = ask_for_name( playground, clock )

			# Run the fade-out transition.
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.ENTER_NAME_TRANSITION["transition_fade_out"], player_name_sprite, True )
			
			return_from_game_status, player_time = run_game( playground, clock )
			if return_from_game_status == "return_from_game_over":
			
				# Nicely fade the game over transition out.
				helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.GAME_OVER_TRANSITION["transition_fade_out"], main_menu )

			elif return_from_game_status == "return_game_won":

				# Nicely fade the game finished transition out.
				helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.GAME_COMPLETED_TRANSITION["transition_fade_out"], main_menu )
							
				# The player has finished / won the game!  Check if he has made it into the highscores.
				helpers.check_highscores( player_name, player_time, constants.HIGHSCORE_DATA["highscore_filename"] )

		elif action == "show_help":
			# Show the help menu.
			show_help_menu( playground, clock )

			# Run the transition.
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.MAIN_MENU_TRANSITION["transition_fade_in"] )
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.MAIN_MENU_TRANSITION["transition_fade_out"], main_menu )				

		elif action == "show_prologue":
			# Show the prologue menu
			show_prologue_menu( playground, clock )

			# Run the transition.
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.MAIN_MENU_TRANSITION["transition_fade_in"] )
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.MAIN_MENU_TRANSITION["transition_fade_out"], main_menu )			

		elif action == "show_highscores":
			# Show the highscore menu
			show_highscore_menu( playground, clock )

			# Run the transition.
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.MAIN_MENU_TRANSITION["transition_fade_in"] )
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.MAIN_MENU_TRANSITION["transition_fade_out"], main_menu )		
						
		elif action == "exit_game":
			break

	pygame.display.quit()


def show_help_menu( playground, clock ):
	"""
	Displays the help menu of the game.

	Parameters
		playground		pygame.Surface
			Everything will be blitted here.
		clock			pygame.time.Clock
			Runs the game at a specified frame-rate.
	"""

	# Create the help menu.
	help_menu = Menu( **constants.HELP_MENU_DATA )

	# Now run the transition.
	helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_in"] )
	helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_out"], help_menu )

	while True:
		events = pygame.event.get()

		# Update the help menu.  'action' will contain a string value if a button of the menu has been pressed.
		action = help_menu.update()

		# And draw the menu.
		playground.blit( help_menu.image, help_menu.rect )

		# Flip the screen so the drawn menu is visible.
		pygame.display.flip()

		# Run everything at a specified frame-rate.
		clock.tick( constants.CLOCK_FPS )

		# Now check if a button has been pressed and which.  If no button is pressed, the following block of code is simply ignored.
		if action == "back":
			# Return to the main menu.
			break


def show_prologue_menu( playground, clock ):
	"""
	Displays the prologue menu of the game.

	Parameters
		playground		pygame.Surface
			Everything will be blitted here.
		clock			pygame.time.Clock
			Runs the game at a specified frame-rate.
	"""

	# Create the prologue menu.
	prologue_menu = Menu( **constants.PROLOGUE_MENU_1_DATA )

	# Now run the transition.
	helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_in"] )
	helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_out"], prologue_menu )

	while True:
		events = pygame.event.get()

		# Update the help menu.  'action' will contain a string value if a button of the menu has been pressed.
		action = prologue_menu.update()

		# And draw the menu.
		playground.blit( prologue_menu.image, prologue_menu.rect )

		# Flip the screen so the drawn menu is visible.
		pygame.display.flip()

		# Run everything at a specified frame-rate.
		clock.tick( constants.CLOCK_FPS )

		# Now check if a button has been pressed and which.  If no button is pressed, the following block of code is simply ignored.
		if action == "back":
			# Return to the main menu.
			break
		elif action == "next":
			# Show the next page of the prologue.
			
			# Create the prologue menu.
			prologue_menu2 = Menu( **constants.PROLOGUE_MENU_2_DATA )

			# Now run the transition.
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_in"] )
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_out"], prologue_menu2 )

			while True:
				events = pygame.event.get()

				# Update the help menu.  'action' will contain a string value if a button of the menu has been pressed.
				action = prologue_menu2.update()

				# And draw the menu.
				playground.blit( prologue_menu2.image, prologue_menu2.rect )

				# Flip the screen so the drawn menu is visible.
				pygame.display.flip()

				# Run everything at a specified frame-rate.
				clock.tick( constants.CLOCK_FPS )

				# Now check if a button has been pressed and which.  If no button is pressed, the following block of code is simply ignored.
				if action == "back":
					# Return to the main menu.
					return
				elif action == "previous":
					# Show the previous page of the prologue.
					break	

			# Now run the transition.
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_in"] )
			helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_out"], prologue_menu )


def show_highscore_menu( playground, clock ):
	"""
	Shows the highscore (menu).

	Parameters
		playground		pygame.Surface
			Everything will be blitted here.
		clock			pygame.time.Clock
			Runs the game at a specified frame-rate.
	"""

	# Highscore test.
	highscore_table = Highscore_Table( **copy.deepcopy( constants.HIGHSCORE_DATA ) )		

	# Create the help menu.
	highscore_menu = Menu( **constants.HIGHSCORE_MENU_DATA )

	# Draw the highscore table onto the highscore menu surface.
	highscore_table.draw( highscore_menu.image )

	# Now run the transition.
	helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_in"] )
	helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.HELP_MENU_TRANSITION["transition_fade_out"], highscore_menu )

	while True:
		events = pygame.event.get()

		# Update the highscore menu.  'action' will contain a string value if a button of the menu has been pressed.
		action = highscore_menu.update()

		# And draw the menu.
		playground.blit( highscore_menu.image, highscore_menu.rect )

		# Draw the highscores
		highscore_table.draw( playground )		

		# Flip the screen so the drawn menu is visible.
		pygame.display.flip()

		# Run everything at a specified frame-rate.
		clock.tick( constants.CLOCK_FPS )

		# Now check if a button has been pressed and which.  If no button is pressed, the following block of code is simply ignored.
		if action == "back":
			# Return to the main menu.
			break


def ask_for_name( playground, clock ):
	"""
	Asks the user for his / her name and returns that name as soon as the user presses <Enter>.
	"""

	user_name = ""
	center = ( constants.WINDOW_DIMENSIONS[0] / 2, constants.WINDOW_DIMENSIONS[1] / 2 + 100 )

	user_name_sprite = Text( user_name, center, font_size=28 )


			# Run the transition.
	trans = helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.ENTER_NAME_TRANSITION["transition_fade_in"] )
#	helpers.run_transition( playground, clock, constants.CLOCK_FPS, constants.MAIN_MENU_TRANSITION["transition_fade_out"], main_menu )		

	running = True
	while running:
		eventlist = pygame.event.get()
		pressed_keys = pygame.key.get_pressed()

		for event in eventlist:
			if event.type == pygame.KEYDOWN:
				if event.key == pygame.K_a:
					user_name += "a"
				if event.key == pygame.K_b:
					user_name += "b"
				if event.key == pygame.K_c:
					user_name += "c"
				if event.key == pygame.K_d:
					user_name += "d"
				if event.key == pygame.K_e:
					user_name += "e"
				if event.key == pygame.K_f:
					user_name += "f"
				if event.key == pygame.K_g:
					user_name += "g"
				if event.key == pygame.K_h:
					user_name += "h"
				if event.key == pygame.K_i:
					user_name += "i"
				if event.key == pygame.K_j:
					user_name += "j"
				if event.key == pygame.K_k:
					user_name += "k"
				if event.key == pygame.K_l:
					user_name += "l"
				if event.key == pygame.K_m:
					user_name += "m"
				if event.key == pygame.K_n:
					user_name += "n"
				if event.key == pygame.K_o:
					user_name += "o"
				if event.key == pygame.K_p:
					user_name += "p"
				if event.key == pygame.K_q:
					user_name += "q"
				if event.key == pygame.K_r:
					user_name += "r"
				if event.key == pygame.K_s:
					user_name += "s"
				if event.key == pygame.K_t:
					user_name += "t"
				if event.key == pygame.K_u:
					user_name += "u"
				if event.key == pygame.K_v:
					user_name += "v"
				if event.key == pygame.K_w:
					user_name += "w"
				if event.key == pygame.K_x:
					user_name += "x"
				if event.key == pygame.K_y:
					user_name += "y"
				if event.key == pygame.K_z:
					user_name += "z"
				if event.key == pygame.K_BACKSPACE:
					user_name = user_name[0:-1]
				if event.key == pygame.K_SPACE:
					user_name += " "
				if event.key == pygame.K_RETURN:
					running = False
				user_name = user_name.upper()					

		# Update the text if necessary.
		user_name_sprite.update( user_name )

		playground.fill( constants.BLACK )
		playground.blit( trans.image, trans.rect )
		playground.blit( user_name_sprite.image, user_name_sprite.rect )
			
		clock.tick( constants.CLOCK_FPS )
		pygame.display.flip()

	return user_name, user_name_sprite
		
if __name__ == "__main__":
	main()
